/**************************** CPP SOURCE FILE ********************************
**
**  Project:    RTCU Communication Hub, Blacklist plug-in
**  Filename:   PLUGIN_MAIN.CPP 
**  Version:    1.00
**  Date:       January 30, 2019
**  
******************************************************************************
**
**  VERSION HISTORY:
**  ----------------
**  1.00 - January 30, 2019 - Michael Olsen
**  - Ported to RCH.
**
*****************************************************************************/


/*****************************************************************************
**
** MODULES USED 
**
*****************************************************************************/
#include "..\interface\gateway_plugin.h"
#include "..\interface\gateway_services.h"
#include "plugin_main.h"
#include "local.h"
#include <string>


/*****************************************************************************
**
**    DEFINITIONS AND MACROS
**
*****************************************************************************/
#define PLUGIN_NAME           "Blacklist"
#define PLUGIN_DESC           "When a login request is received from a client, the blacklist plug-in will lookup the requested Node ID in the blacklist, and if the Node ID is found, the login is denied.\n\nThe blacklist plug-in reads the list of denied Node ID's from an UTF-8 encoded XML file. The list will be updated at a user defined time interval.\n\nThe file must follow this format:\n\n<blacklist>\n<node>\n<id>2000001</id>\n<text>Message shown in log on rejection</text>\n</node>\n<node>\n<id>2000002</id>\n<text />\n</node>\n</blacklist>"
#define PLUGIN_VER            "1.01"
#define PLUGIN_CONFIG         "<configuration><property><type>file</type><name>File</name><default /><description>Path to the UTF-8 encoded XML file that contains the Node ID's to prevent login for.\nNote that files on subst drives cannot be read.</description><ext>*.xml</ext><filter>XML files|*.xml|All files|*.*||</filter></property><property><type>num</type><name>Refresh interval</name><default>10</default><description>The number of minutes to wait before updating the list of denied Node ID's</description><min>1</min><max>30</max></property></configuration>"


/****************************************************************************
**
**    TYPEDEFS AND STRUCTURES
**
****************************************************************************/


/*****************************************************************************
**
** EXPORTED VARIABLES 
**
*****************************************************************************/


/*****************************************************************************
**
** GLOBAL VARIABLES 
**
*****************************************************************************/
static gwpf_func_invoke              invoker = nullptr;


/*****************************************************************************
**
** LOCAL FUNCTIONS 
**
*****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// This function is called by the RCH to read the required text info.
//
// Input:
//    Text          - The index of the text info to read.
//
// Output:
//    None.
//
// Returns:
//    None.
//
/////////////////////////////////////////////////////////////////////////////
static const char* GWPF_ReadText(const int Text)
{
  // Return selected text
  switch (Text) {
    case GWPF_TEXT_NAME:        return PLUGIN_NAME;
    case GWPF_TEXT_DESCRIPTION: return PLUGIN_DESC;
    case GWPF_TEXT_VERSION:     return PLUGIN_VER;
    case GWPF_TEXT_CONFIG:      return PLUGIN_CONFIG;
  }

  // Unknown text
  return "";
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by the RCH to close and cleanup the plug-in.
//
// Input:
//    None.
//
// Output:
//    None.
//
// Returns:
//    None.
//
/////////////////////////////////////////////////////////////////////////////
static void GWPF_ExitPlugin(void)
{
  // Stop
  StopBlacklist();
}


/*****************************************************************************
**
** EXPORTED FUNCTIONS 
**
*****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// This function will write an entry into the RCH log.
//
// Input:
//    type          - The type of log entry.
//                    1 = Error, 2 = Event, 3 = detailed event
//    text          - Pointer to the zero terminated UTF-8 string to enter.
//
// Output:
//    None.
//
// Returns:
//    None.
//
/////////////////////////////////////////////////////////////////////////////
void write_log(const int type, const char *text)
{
  // Var
  gwpf_service_log_params log_;

  // Verify
  if (type < 1 || type > 3) return;
  if (text == nullptr || strlen(text) == 0) return;
  if (invoker == nullptr) return;

  // Insert text to RCH log
  log_.type    = type;
  log_.message = text;
  invoker(PLUGIN_NAME, GWPF_SERVICE_LOG, &log_);
}

/////////////////////////////////////////////////////////////////////////////
// This function is the plug-in entry point and is called by the RCH to
// initialize the plug-in.
//
// Input:
//    Services      - Pointer to the RCH service structure.
//
// Output:
//    None.
//
// Returns:
//    None.
//
/////////////////////////////////////////////////////////////////////////////
gwpf_func_exit GWPF_InitPlugin(const gwpf_gateway_services *Services)
{
  // Var
  gwpf_api_version              api_ver;
  gwpf_readtext_params          reader_setup;
  gwpf_service_onlogon_params   sub_onlogon;
  int                           init = 1;

  // Store RCH services
  invoker               = Services->invoker;

  // Register Plugin with RCH
  api_ver.major         = 1;
  api_ver.minor         = 0;
  if (Services->registrar(PLUGIN_NAME, &api_ver)) {
    // Plugin rejected
    return GWPF_ExitPlugin;
  }

  // Register text reader function
  reader_setup.func     = GWPF_ReadText;
  Services->subscriber(PLUGIN_NAME, GWPF_SERVICE_READTEXT, &reader_setup);

  // Configure plugin
  if (Services->configuration != nullptr) {
    init = InitBlacklist(Services->configuration);
  }

  // Subscribe
  sub_onlogon.func = OnLogon;
  Services->subscriber(PLUGIN_NAME, GWPF_SERVICE_LOGON, &sub_onlogon);

  // Insert entry into RCH log
  switch (init) {
    case 0: write_log(2, "Blacklist plug-in 1.01 loaded and ready."); break;
    case 1: write_log(1, "Blacklist plug-in load error: Not configured."); break;
    case 2: write_log(1, "Blacklist plug-in load error: No file specified."); break;
    case 3: write_log(2, "Blacklist plug-in 1.01 loaded."); break;
    case 4: write_log(2, "Blacklist plug-in 1.01 loaded."); break;
  }

  // Completed
  return GWPF_ExitPlugin;
}


/****************************************************************************
**
**    EOF 
**
****************************************************************************/
