/**************************** CPP SOURCE FILE ********************************
**
**  Project:    RTCU Communication Hub, Heartbeat plug-in
**  Filename:   PLUGIN_MAIN.CPP 
**  Version:    1.00
**  Date:       June 23, 2020
**  
******************************************************************************
**
**  VERSION HISTORY:
**  ----------------
**  1.00 - June 23, 2020 - Michael Olsen
**  - The functions required for the RCH Plugin API is implemented.
**
*****************************************************************************/


/*****************************************************************************
**
** MODULES USED 
**
*****************************************************************************/
#include "..\interface\gateway_plugin.h"
#include "..\interface\gateway_services.h"
#include "plugin_main.h"
#include "local.h"
#include <string>


/*****************************************************************************
**
**    DEFINITIONS AND MACROS
**
*****************************************************************************/
#define PLUGIN_NAME           "Heartbeat"
#define PLUGIN_DESC           "This plugin is used to monitor the heartbeat of connected RTCU devices."
#define PLUGIN_VER            "1.00"
#define PLUGIN_CONFIG         "<configuration></configuration>"


/****************************************************************************
**
**    TYPEDEFS AND STRUCTURES
**
****************************************************************************/


/*****************************************************************************
**
** EXPORTED VARIABLES 
**
*****************************************************************************/


/*****************************************************************************
**
** GLOBAL VARIABLES 
**
*****************************************************************************/
static gwpf_func_invoke              invoker = nullptr;


/*****************************************************************************
**
** LOCAL FUNCTIONS 
**
*****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// This function is called by the RCH to read the required text info.
//
// Input:
//    Text          - The index of the text info to read.
//
// Output:
//    None.
//
// Returns:
//    None.
//
/////////////////////////////////////////////////////////////////////////////
static const char* GWPF_ReadText(const int Text)
{
  // Return selected text
  switch (Text) {
    case GWPF_TEXT_NAME:        return PLUGIN_NAME;
    case GWPF_TEXT_DESCRIPTION: return PLUGIN_DESC;
    case GWPF_TEXT_VERSION:     return PLUGIN_VER;
    case GWPF_TEXT_CONFIG:      return PLUGIN_CONFIG;
  }

  // Unknown text
  return "";
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by the RCH to close and cleanup the plug-in.
//
// Input:
//    None.
//
// Output:
//    None.
//
// Returns:
//    None.
//
/////////////////////////////////////////////////////////////////////////////
static void GWPF_ExitPlugin(void)
{
    // None of the functionality in this plugin
    // needs to be stopped 
}


/*****************************************************************************
**
** EXPORTED FUNCTIONS 
**
*****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// This function will write an entry into the RCH log.
//
// Input:
//    type          - The type of log entry.
//                    1 = Error, 2 = Event, 3 = detailed event
//    text          - Pointer to the zero terminated UTF-8 string to enter.
//
// Output:
//    None.
//
// Returns:
//    None.
//
/////////////////////////////////////////////////////////////////////////////
void write_log(const int type, const char *text)
{
  // Var
  gwpf_service_log_params log_;

  // Verify
  if (type < 1 || type > 3) return;
  if (text == nullptr || strlen(text) == 0) return;
  if (invoker == nullptr) return;

  // Insert text to RCH log
  log_.type    = type;
  log_.message = text;
  invoker(PLUGIN_NAME, GWPF_SERVICE_LOG, &log_);
}

/////////////////////////////////////////////////////////////////////////////
// This function is the plug-in entry point and is called by the RCH to
// initialize the plug-in.
//
// Input:
//    Services      - Pointer to the RCH service structure.
//
// Output:
//    None.
//
// Returns:
//    None.
//
/////////////////////////////////////////////////////////////////////////////
gwpf_func_exit GWPF_InitPlugin(const gwpf_gateway_services *Services)
{
  // Var
  gwpf_api_version              api_ver;
  gwpf_readtext_params          reader_setup;
  gwpf_service_heartbeat_params sub_onheartbeat;
  int                           init = 1;

  // Store RCH services
  invoker               = Services->invoker;

  // Register Plugin with RCH
  api_ver.major         = 1;
  api_ver.minor         = 0;
  if (Services->registrar(PLUGIN_NAME, &api_ver)) {
    // Plugin rejected
    return GWPF_ExitPlugin;
  }

  // Register text reader function
  reader_setup.func     = GWPF_ReadText;
  Services->subscriber(PLUGIN_NAME, GWPF_SERVICE_READTEXT, &reader_setup);

  // Subscribe
  sub_onheartbeat.func = OnHeartbeat;
  Services->subscriber(PLUGIN_NAME, GWPF_SERVICE_HEARTBEAT, &sub_onheartbeat);

  // Insert entry into RCH log
  write_log(2, "Heartbeat plug-in 1.00 loaded and ready.");

  // Completed
  return GWPF_ExitPlugin;
}


/****************************************************************************
**
**    EOF 
**
****************************************************************************/
