
#ifndef _GATEWAY_PLUGIN_H_
#define _GATEWAY_PLUGIN_H_

#ifdef __cplusplus
  extern "C" {
#endif

/////////////////////////////////////////////////////////////////////////////
// This structure holds the Gateway Plugin Framework API version.
/////////////////////////////////////////////////////////////////////////////
typedef struct gwpf_api_version {
   int                        major;
   int                        minor;
} gwpf_api_version;

/////////////////////////////////////////////////////////////////////////////
// This is the function pointer used by the plugin to subscribe to an event
// service offered by the Gateway.
//
// Input:
//    PluginName        - Zero terminated UTF8 string with the name of the
//                        plugin that is registering.
//
//    ServiceID         - The ID of the event service to subscribe to.
//
//    ServiceParams     - Pointer to structure containing the parameters
//                        required for the plugin to subscribe to the service.
//
// Output:
//    None.
//
// Returns:
//    0                 - Success.
//    1                 - Plug-in is not found.
//    2                 - Plug-in is disabled.
//    3                 - Unknown service.
//    4                 - Invalid parameters.
//    5                 - Plug-in have already subscribed to the service.
//
/////////////////////////////////////////////////////////////////////////////
typedef int (__cdecl *gwpf_func_subscribe)(const char *PluginName,
                                           const int  ServiceID,
                                           void       *ServiceParams);

/////////////////////////////////////////////////////////////////////////////
// This is the function pointer used by the plugin to register itself with
// the Gateway.
// If the registration fails, the Gateway will perform a gracefull close of
// the Plugin with the ExitPlugin function.
//
// Input:
//    PluginName        - Zero terminated UTF8 string with the name of the
//                        plugin that is registering.
//
//    Version           - Pointer to structure containing the version of the
//                        API used by the Plugin.
//
// Output:
//    None.
//
// Returns:
//    0                 - Plug-in is registered successfully.
//    1                 - Plug-in is not found.
//    2                 - Plug-in is incompatible with Gateway.
//
/////////////////////////////////////////////////////////////////////////////
typedef int (__cdecl *gwpf_func_register)(const char       *PluginName,
                                          gwpf_api_version *Version);

/////////////////////////////////////////////////////////////////////////////
// This is the function pointer used by the plugin to invoke the services
// offered by the Gateway.
//
// Input:
//    PluginName        - Zero terminated UTF8 string with the name of the
//                        plug-in.
//
//    ServiceID         - The ID of the service to invoke.
//
//    ServiceParams     - Pointer to structure containing the parameters
//                        required by the service.
//
// Output:
//    None.
//
// Returns:
//    0                 - Success.
//    1                 - Plug-in is not found.
//    2                 - Plug-in is disabled.
//    3                 - Unknown service.
//    4                 - Invalid parameters.
//    Other error codes as per individual service.
//
/////////////////////////////////////////////////////////////////////////////
typedef int (__cdecl *gwpf_func_invoke)(const char *PluginName,
                                        const int  ServiceID,
                                        void       *ServiceParams);

/////////////////////////////////////////////////////////////////////////////
// This structure holds the parameters necessary for the Plugin to
// register with the Gateway.
/////////////////////////////////////////////////////////////////////////////
typedef struct gwpf_gateway_services {
  gwpf_api_version            version;
  gwpf_func_register          registrar;
  gwpf_func_subscribe         subscriber;
  gwpf_func_invoke            invoker;
  const char                  *configuration;
} gwpf_gateway_services;

/////////////////////////////////////////////////////////////////////////////
// This is the function pointer used by the Gateway to close a plugin.
//
// Input:
//    None.
//
// Output:
//    None.
//
// Returns:
//    None.
//
/////////////////////////////////////////////////////////////////////////////
typedef void (__cdecl *gwpf_func_exit)(void);

/////////////////////////////////////////////////////////////////////////////
// This is the function pointer used by the Gateway to initialize the Plugin.
// The implementation of this function must be named GWPF_InitPlugin.
//
// Input:
//    Services          - Pointer to structure containing the parameters
//                        required by the plugin to work with the Gateway.
//
// Output:
//    None.
//
// Returns:
//    Pointer to function for closing the plugin.
//
/////////////////////////////////////////////////////////////////////////////
typedef gwpf_func_exit (__cdecl *gwpf_func_init)(const gwpf_gateway_services *Services);


//
// Text reader
//
#define GWPF_SERVICE_READTEXT           1
#define GWPF_TEXT_NAME                  1
#define GWPF_TEXT_DESCRIPTION           2
#define GWPF_TEXT_VERSION               3
#define GWPF_TEXT_CONFIG                4

/////////////////////////////////////////////////////////////////////////////
// This is the function pointer used by the Gateway to read static text
// from the plugin.
//
// Input:
//    Text              - The ID of the statis text to read.
//
// Output:
//    None.
//
// Returns:
//    Pointer to zero terminated UTF-8 string with the static text.
//
/////////////////////////////////////////////////////////////////////////////
typedef const char* (__cdecl *gwpf_func_readtext)(const int Text);

/////////////////////////////////////////////////////////////////////////////
// This structure holds the parameters for registering the Plugin text reader
// function with the Gateway.
//
// Input:
//    func              - Pointer to function that is called to read text.
//
// Output:
//    None.
//
// Returns:
//    No additional errors.
//
/////////////////////////////////////////////////////////////////////////////
typedef struct gwpf_readtext_params {
   gwpf_func_readtext func;
} gwpf_readtext_params;


#ifdef  __cplusplus
   }
#endif

#endif  /* _GATEWAY_PLUGIN_H_ */
